#include "ModuleBase.h"
#include <algorithm>

namespace mdd {

    Input::Input(const std::string& type, const json& default_value,
                 const std::function<bool(const json&)>& verification) {
        this->type = type;
        value = default_value;
        this->verification = std::move(verification);
    }

    json& ModuleBase::setInputDefaultValue(int handle) {
        return _inputs[handle].value;
    }


    const json& ModuleBase::getInputValue(int handle) {
        if (_inputs[handle].output == nullptr)
            return _inputs[handle].value;

        return _inputs[handle].output->getValue();

    }

    json& ModuleBase::setOutputValue(int handle) {
        static json empty;
        if (_outputs.size() <= handle)
            return empty;

        return _outputs[handle]->getValueInternal();
    }


    std::vector<std::string> ModuleBase::getInputs() {
        std::vector<std::string> ret;
        for (auto &input  : _inputs) {
            ret.push_back(input.type);
        }
        return ret;
    }

    std::vector<std::string> ModuleBase::getOutputs() {
        std::vector<std::string> ret;
        for (auto &output  : _outputs) {
            ret.push_back(output->getType());
        }
        return ret;
    }

    int ModuleBase::addInput(const std::string& type, const json& value,
                             const std::function<bool(const json&)>& verification) {
        _inputs.emplace_back(Input(type, value, verification));
        return _inputs.size() - 1;
    }

    bool ModuleBase::connectInput(int handle, std::shared_ptr<IOutput> output) {
        if (_inputs[handle].verification(output->getValue())) {
            _inputs[handle].output = output;
            return true;
        }
        return false;
    }

    std::shared_ptr<IOutput> ModuleBase::getOutput(int handle) {
        return _outputs[handle];
    }

    int ModuleBase::addOutput(const std::string& type, const json& initial) {
        _outputs.push_back(std::make_shared<Output>( type, initial));
        return _outputs.size()-1;
    }
}